// Generated by N4JS transpiler; for copyright see original N4JS source file.

import 'n4js-runtime'

export class DispatchQueue extends N4Object {
	constructor() {
		super();
		this.listeners = undefined;
		this.lastIndex = 0;
		this.lastInsertionOrder = 0;
		this.shouldHalt = false;
		this.isDispatching = false;
		this.listeners = [
			null
		];
	}
	has(listener, context) {
		for(var i = 1;i <= this.lastIndex;i++) {
			if ((this.listeners[i].listener == listener) && (this.listeners[i].context == context)) {
				return true;
			}
		}
		return false;
	}
	isEmpty() {
		return (this.lastIndex < 1);
	}
	add(listener, listenerContext, priority=0) {
		this.addListener(listener, listenerContext, priority, false);
	}
	addOnce(listener, listenerContext, priority) {
		this.addListener(listener, listenerContext, priority, true);
	}
	remove(listener, context) {
		var listenerIdx = this.listeners.findIndex((elem, index, array)=>{
			return ((elem !== null) && (elem.listener === listener) && (elem.context === context));
		});
		if (listenerIdx >= 0) {
			this.removeListenerAt(listenerIdx);
		}
	}
	removeAll() {
		while(this.listeners.length > 1) {
			this.listeners.pop();
		}
		this.lastIndex = 0;
		this.lastInsertionOrder = 0;
		return this;
	}
	getNumListeners() {
		return this.lastIndex;
	}
	halt() {
		this.shouldHalt = true;
		return this;
	}
	async dispatch(params) {
		this.isDispatching = true;
		var listeners = this.getListOfListeners();
		for(var i = 0;i < listeners.length;i++) {
			var current = listeners[i];
			if (!this.shouldHalt) {
				await Promise.resolve(current.listener.apply(current.listenerContext, params));
				if (current.applyOnlyOnce) {
					this.remove(current.listener, current.listenerContext);
				}
			}
		}
		this.isDispatching = false;
		this.shouldHalt = false;
	}
	getListOfListeners() {
		var oldListeners = this.listeners.slice();
		var oldLastIndex = this.lastIndex;
		var oldLastInsertionOrder = this.lastInsertionOrder;
		var result = new Array();
		for(var i = 1;i <= oldLastIndex;i++) {
			var current = this.removeListenerWithHighestPriority();
			result.push({
				listener: current.listener,
				listenerContext: current.context,
				priority: current.priority,
				insertionOrder: current.insertionOrder,
				applyOnlyOnce: current.applyOnlyOnce
			});
		}
		this.lastIndex = oldLastIndex;
		this.lastInsertionOrder = oldLastInsertionOrder;
		while(this.listeners.length > 1) {
			this.listeners.pop();
		}
		for(i = 1;i <= this.lastIndex;i++) {
			this.listeners.push(oldListeners[i]);
		}
		return result;
	}
	dispose() {
		while(this.listeners.length) {
			this.listeners.pop();
		}
		this.listeners = undefined;
		this.lastIndex = 0;
		this.lastInsertionOrder = 0;
	}
	toString() {
		return this.getListOfListeners().toString();
	}
	addListener(listener, listenerContext, priority, applyOnlyOnce) {
		if (this.isDispatching) {
			throw "Adding a listener while dispatching is forbidden.";
		}
		this.lastInsertionOrder++;
		this.lastIndex++;
		this.listeners.push(new ListenerDS({
			listener: listener,
			context: listenerContext,
			priority: priority,
			applyOnlyOnce: applyOnlyOnce,
			insertionOrder: this.lastInsertionOrder
		}));
		this.shiftUp(this.lastIndex);
	}
	removeListenerAt(index) {
		var listenerToBeRemoved = this.listeners[index];
		this.listeners[index] = this.listeners[this.lastIndex];
		var parentIndex = Math.floor(index / 2);
		if ((parentIndex >= 1) && (this.listeners[index].compare(this.listeners[parentIndex]) > 0)) {
			this.shiftUp(index);
		} else {
			this.shiftDown(index);
		}
		this.listeners.pop();
		this.lastIndex--;
		return listenerToBeRemoved;
	}
	removeListenerWithHighestPriority() {
		if (this.isEmpty()) {
			return null;
		}
		return this.removeListenerAt(1);
	}
	shiftDown(index) {
		if (index >= this.lastIndex) {
			return;
		}
		var nextIndex = index;
		var leftChildIndex = 2 * index;
		var rightChildIndex = 2 * index + 1;
		if ((leftChildIndex < this.lastIndex) && (this.listeners[index].compare(this.listeners[leftChildIndex]) < 0)) {
			index = leftChildIndex;
		}
		if ((rightChildIndex < this.lastIndex) && (this.listeners[index].compare(this.listeners[rightChildIndex]) < 0)) {
			index = rightChildIndex;
		}
		if (nextIndex != index) {
			this.swap(index, nextIndex);
			this.shiftDown(nextIndex);
		}
	}
	swap(index1, index2) {
		var tmp = this.listeners[index1];
		this.listeners[index1] = this.listeners[index2];
		this.listeners[index2] = tmp;
	}
	shiftUp(index) {
		if (index <= 1) {
			return;
		}
		var parentIndex = Math.floor(index / 2);
		if (this.listeners[index].compare(this.listeners[parentIndex]) > 0) {
			this.swap(index, parentIndex);
			this.shiftUp(parentIndex);
		}
	}
	static get n4type() {
		return $getReflectionForClass(this, '["DispatchQueue","org/eclipse/n4js/mangelhaft/types/DispatchQueue","org.eclipse.n4js.mangelhaft",["f.listeners","f.lastIndex","f.lastInsertionOrder","f.shouldHalt","f.isDispatching"]]');
	}
}
class ListenerDS extends N4Object {
	constructor(spec) {
		super();
		const $specObj = spec || {};
		({
			listener: this.listener,
			context: this.context,
			priority: this.priority,
			applyOnlyOnce: this.applyOnlyOnce,
			insertionOrder: this.insertionOrder
		} = $specObj);
	}
	compare(otherListener) {
		if (this.priority == otherListener.priority) {
			if (this.insertionOrder < otherListener.insertionOrder) {
				return 1;
			} else {
				return -1;
			}
		} else if (this.priority > otherListener.priority) {
			return 1;
		} else {
			return -1;
		}
	}
	static get n4type() {
		return $getReflectionForClass(this, '["ListenerDS","org/eclipse/n4js/mangelhaft/types/DispatchQueue","org.eclipse.n4js.mangelhaft",["f.listener","f.context","f.priority","f.applyOnlyOnce","f.insertionOrder"]]');
	}
}
//# sourceMappingURL=DispatchQueue.map
